/******************************************************************************
 *
 * macros.h - endianess handling macros. Part of the DirTools project.
 *
 * $Revision: 1.2 $
 *
 * Copyright (c) 2001-2004 Olivier Castan
 *
 * This program/include file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program/include file is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (in the main directory of the Linux-NTFS
 * distribution in the file COPYING); if not, write to the Free Software
 * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * format : indent -br -ce -cdw -nut -i5
 *
 *****************************************************************************/

#ifndef _MACROS_H_
#define _MACROS_H_ 1

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdint.h>
#ifndef _WIN32
#include "wincompat.h"
#endif /* _WIN32 */

#ifndef min
#define min(a,b)  ((a) < (b) ? (a) : (b))
#define max(a,b)  ((a) < (b) ? (b) : (a))
#endif

#define IS_MAGIC(a,b)  (*(uint32_t *)(a)==*(uint32_t *)(b))
#define IS_MFT_RECORD(a)  IS_MAGIC((a),"FILE")

#define IS_NTFS_VOL(a) (*(int64_t *)(a)==*(int64_t *)"NTFS    ")

#ifdef WORDS_BIGENDIAN
#define LE16_TO_CPU(a)  ((((a) & 0xFF) << 8) | ((a) >> 8))
#define LE32_TO_CPU(a)  ((LE16_TO_CPU((a) & 0xFFFF) << 16) | \
                          LE16_TO_CPU((a) >> 16))
#define LE64_TO_CPU(a)  ((LE32_TO_CPU((a) & 0xFFFFFFFF) << 32) | \
                          LE32_TO_CPU((a) >> 32))
#else /* WORDS_BIGENDIAN */
#define LE16_TO_CPU(a) (a)
#define LE32_TO_CPU(a) (a)
#define LE64_TO_CPU(a) (a)
#endif /* WORDS_BIGENDIAN */

#define DT_GETU8(p)  (*(unsigned char *)(p))
#define DT_GETU16(p)  ((uint16_t)LE16_TO_CPU(*(uint16_t *)(p)))
#define DT_GETU24(p) (((uint32_t)DT_GETU16(p)) | \
                      (((uint32_t)DT_GETU8((unsigned char *)(p) + 2))<< 16))
#define DT_GETU32(p)  ((uint32_t)LE32_TO_CPU(*(uint32_t *)(p)))
#define DT_GETU40(p)  (((uint64_t)DT_GETU32(p)) | \
                       (((uint64_t)DT_GETU8((unsigned char *)(p) + 4))<< 32))
#define DT_GETU48(p)  (((uint64_t)DT_GETU32(p)) | \
                       (((uint64_t)DT_GETU16((unsigned char *)(p) + 4))<< 32))
#define DT_GETU56(p)  (((uint64_t)DT_GETU32(p)) | \
                       (((uint64_t)DT_GETU24((unsigned char *)(p) + 4))<< 32))
#define DT_GETU64(p)  ((uint64_t)LE64_TO_CPU(*(uint64_t *)(p)))

#define DT_GETS8(p)  (*(char *)(p))
#define DT_GETS16(p) ((int16_t)LE16_TO_CPU(*(int16_t *)(p)))
#define DT_GETS24(p) (((int32_t)DT_GETU16((p))) | \
                      (((int32_t)DT_GETS8((char *)(p) + 2))<< 16))
#define DT_GETS32(p) ((int32_t)LE32_TO_CPU(*(int32_t *)(p)))
#define DT_GETS40(p) (((int64_t)DT_GETU32(p)) | \
                      (((int64_t)DT_GETS8((char *)(p) + 4))<< 32))
#define DT_GETS48(p) (((int64_t)DT_GETU32(p)) | \
                      (((int64_t)DT_GETS16((char *)(p) + 4))<< 32))
#define DT_GETS56(p) (((int64_t)DT_GETU32(p)) | \
                      (((int64_t)DT_GETS24((char *)(p) + 4))<< 32))
#define DT_GETS64(p) ((int64_t)LE64_TO_CPU(*(int64_t *)(p)))

#endif /* _MACROS_H_ */
