/******************************************************************************
 *
 * utility.c - program options support. Part of the DirTools project.
 *
 * $Revision: 1.1.1.1 $
 *
 * Copyright (c) 2001-2004 Olivier Castan
 *
 * This program/include file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program/include file is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (in the main directory of the Linux-NTFS
 * distribution in the file COPYING); if not, write to the Free Software
 * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * format : indent -br -ce -cdw -nut -i5
 *
 *****************************************************************************/

#ifndef _UTILITY_H_
#define _UTILITY_H_ 1

#include <stdlib.h>
#include <stdint.h>
#ifdef _WIN32
#include <windows.h>
#else /* _WIN32 */
#include "wincompat.h"
#endif /* _WIN32 */

#include "dt_error.h"

#define MANDATORYFILENAME(source, dest, msg) \
        do { \
                if (source) { \
                        (dest) = (source); \
                } else { \
                        Alarm(TA_PROG_FATAL, \
                              "%s: missing %s filename", \
                              argv[0], \
                              (msg)); \
                        _fcloseall(); \
                        exit(EXIT_FAILURE); \
                } \
        } while(0)


#define OUTPUTFILE(path, descriptor, msg) \
        do{ \
                if (path) { \
                        (descriptor) = CreateOutputFile(path); \
                        if (! (descriptor)) { \
                                Alarm(TA_PROG_FATAL, \
                                      "%s: unable to create %s file %s", \
                                      argv[0], \
                                      (msg), \
                                      (path)); \
                                _fcloseall(); \
                                exit(EXIT_FAILURE); \
                        } \
                } \
        } while(0)


FILE *CreateOutputFile (const char *);
uint64_t integer_arg (const char *);

#endif /* _UTILITY_H_ */
